package com.persagy.apm.energyalarmstarter.alarmengine.jms;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Sets;
import com.persagy.apm.energyalarmstarter.alarmdata.mq.model.DmpMessage;
import com.persagy.apm.energyalarmstarter.alarmengine.redis.RedisUtil4EngineStater;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.HashSet;
import java.util.List;

/**
 * @description:报警定义消息通知
 * @author:LuoGuangyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/10/20 002016:30
 * @version:V1.0
 **/
@Slf4j
@Configuration
public class JmsConfig {
    /**
     * 报警记录锁前缀
     */
    private static final String alarmRecordLockPrefix = "ALARM_RECORD_LOCK:";
    /**
     * 子系统可处理的报警系统
     */
    @Value("#{'${alarm.category.list}'.split(',')}")
    private List<String> categoryList;

    /**
     * NettyAlarmMsgBaseHandler本身不进行ioc注入，这里注入的是他的子类。
     * 子类在实际项目中创建，starter中没有实例。
     */
    @Autowired
    private AlarmEngineMsgHandler msgHandler;

    @Autowired
    private RedisUtil4EngineStater redisUtil;

    /**
     * 交换机
     */
    private final String exchange = "exchange-alarm-service";

    /**
     * 报警条件路由键
     */
    private final String alarmConditionRoutingKey = "alarm-condition-routing-key";

    /**
     * 报警对象与报警条件关联关系路由键
     */
    private final String objConditionRelRoutingKey = "obj-condition-rel-routing-key";

    /**
     * 报警定义路由键
     */
    private static final String alarmConfigRoutingKey = "alarm-config-routing-key";

    /**
     * 报警记录 路由键
     */
    private final String alarmRecordRoutingKey = "alarm-msg-routing-key";

    /**
     * 报警条件队列
     */
    private final String conditionQueue = "condition-queue";

    /**
     * 报警定义队列(数据中台设备的报警条件)
     */
    private final String alarmConfigQueue = "alarm-config-queue";

    /**
     * 设备与报警条件关联关系队列
     */
    private final String objConditionRelQueue = "obj-condition-rel-queue";
    /**
     * 报警记录队列
     */
    private final String alarmRecordQueue = "alarm-msg-queue";

    @Bean
    public Queue conditionQueue() {
        return new Queue(conditionQueue, true);
    }

    @Bean
    public Queue objConditionRelQueue() {
        return new Queue(objConditionRelQueue, true);
    }

    @Bean
    public Queue alarmConfigQueue() {
        return new Queue(alarmConfigQueue, true);
    }

    @Bean
    public Queue alarmRecordQueue() {
        return new Queue(alarmRecordQueue, true);
    }

    @Bean
    public TopicExchange alarmServiceExchange() {
        return new TopicExchange(exchange);
    }

    @Bean
    public Binding conditionBinding() {
        return BindingBuilder.bind(conditionQueue()).to(alarmServiceExchange()).with(alarmConditionRoutingKey);
    }

    @Bean
    public Binding objConditionRelBinding() {
        return BindingBuilder.bind(objConditionRelQueue()).to(alarmServiceExchange()).with(objConditionRelRoutingKey);
    }

    @Bean
    public Binding alarmConfigBinding() {
        return BindingBuilder.bind(alarmConfigQueue()).to(alarmServiceExchange()).with(alarmConfigRoutingKey);
    }

    @Bean
    public Binding alarmRecordBinding() {
        return BindingBuilder.bind(alarmRecordQueue()).to(alarmServiceExchange()).with(alarmRecordRoutingKey);
    }

    /**
     * 处理业务中台的消息
     *
     * @author lixing
     * @version V1.0 2021/11/19 10:43 下午
     */
    @RabbitHandler
    @RabbitListener(queues = {conditionQueue, objConditionRelQueue})
    public void dealAlarmServiceMsg(DmpMessage msg, Channel channel, Message message) throws Exception {
        try {
            log.debug("============================== Receive:" + msg);
            if (JmsTypeEnum.NEW_CONDITION.getType().equals(msg.getType())) {
                msgHandler.syncNewCondition(msg);
            }
            if (JmsTypeEnum.UPDATE_CONDITION.getType().equals(msg.getType())) {
                msgHandler.syncUpdatedCondition(msg);
            }
            if (JmsTypeEnum.DELETE_CONDITION.getType().equals(msg.getType())) {
                msgHandler.syncDeletedCondition(msg);
            }
            if (JmsTypeEnum.NEW_OBJ_CONDITION_REL.getType().equals(msg.getType())) {
                msgHandler.syncNewObjConditionRelList(msg);
            }
            if (JmsTypeEnum.DELETE_OBJ_CONDITION_REL.getType().equals(msg.getType())) {
                msgHandler.syncDeletedObjConditionRelList(msg);
            }
            if (JmsTypeEnum.UPDATE_OBJ_CONDITION_REL.getType().equals(msg.getType())) {
                msgHandler.syncUpdateObjConditionRelList(msg);
            }
        } catch (Exception e) {
            log.error("报警条件消息消费失败，{}", e.getMessage());
        }
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
    }

    /**
     * 处理报警引擎报警消息
     *
     * @author lixing
     * @version V1.0 2021/11/20 2:19 下午
     */
    @RabbitHandler
    @RabbitListener(queues = {alarmRecordQueue})
    public void dealAlarmRecordMsg(DmpMessage msg, Channel channel, Message message) throws Exception {
        try {
            // 根据报警消息获取报警系统
            String alarmCategory = msgHandler.getAlarmCategory(msg);
            if (StringUtils.isBlank(alarmCategory)) {
                return;
            }
            HashSet<String> categorySet = Sets.newHashSet(categoryList);
            if (!categorySet.contains(alarmCategory)) {
                // 如果子系统不能处理此报警系统的消息，将消息扔回消息队列
                channel.basicNack(message.getMessageProperties().getDeliveryTag(), false, true);
                return;
            }

            log.debug("============================== Receive:" + msg);
            if (JmsAlarmRecordEnum.NEW_ALARM_RECORD.getType().equals(msg.getType())) {
                msgHandler.createAlarmRecordAndSendRecordId(msg);
            }
            if (JmsAlarmRecordEnum.UPDATE_ALARM_RECORD.getType().equals(msg.getType())) {
                msgHandler.updateAlarmRecord(msg);
            }
            if (JmsAlarmRecordEnum.CONTINUE_ALARM.getType().equals(msg.getType())) {
                msgHandler.alarmContinue(msg);
            }
            //解锁
            JSONObject data = msg.getExts();
            redisUtil.unlock(alarmRecordLockPrefix + data.getString("id"));
        } catch (Exception e) {
            log.error("报警记录消息消费失败，{}", e.getMessage());
        }
        channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
    }

    /**
     * 数据中台消息处理
     *
     * @author lixing
     * @version V1.0 2021/11/20 2:20 下午
     */
    @RabbitHandler
    @RabbitListener(queues = {alarmConfigQueue})
    public void dealDmpAlarmMsg(DmpMessage msg, Channel channel, Message message) {
        try {
            log.debug("============================== Receive:" + msg);
            //报警定义变化
            if (JmsTypeEnum.ALARM_CONFIGS_CHANGE.getType().equals(msg.getType())) {
                log.debug("================收到一条报警定义变化通知==============");
                try {
                    msgHandler.incrementSyncAlarmConfig(msg);
                } catch (Exception e) {
                    log.error("error", e);
                }
            }
            channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
        } catch (Exception e) {
            log.error("消息消费失败，{}", e.getMessage());
        }
    }
}
